// A server browser for Armagetron
// Copyright (C) 2010  DLH
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

function truncate(s, len) {
    if (s.length > len)
        return s.substring(0, len);
    return s
}

function default_table_sorting() {
    return [[1, 1], [0,0]];
}

function current_primary_table_sorting() {
    return $("#servers").get(0).config.sortList[0];
}

function strip_colors(s) {
    return s.replace(/0x.{6}/g, "");
}

function server_node(host, port) {
    return $("tr[host='" + host + "'][port='" + port + "']");
}

function ensure_http(uri) {
    if (!/http:\/\//.test(uri))
        uri = "http://" + uri;
    return uri;
}

function move_empty_servers_box() {
    var sorting = current_primary_table_sorting();
    
    // We are sorting by users primarily
    if (sorting && sorting[0] == 1 && sorting[1] == 1) {
        $("#empty_servers_box").insertBefore("#servers tbody tr.empty_server:first");
    }
    else {
        $("#empty_servers_box").appendTo("#servers tbody");
    }
}

function toggle_empty_servers() {
    var empty_box = $("#empty_servers_box");
    var servers = $("#servers > tbody > tr.empty_server");
    
    if (empty_box.hasClass("empty_servers_hidden")) {
        servers.removeClass("hide");
        $("#empty_servers_box_action").text("Hide");
        empty_box.removeClass("empty_servers_hidden");
        empty_box.addClass("empty_servers_visible");
    }
    else {
        servers.addClass("hide");
        $("#empty_servers_box_action").text("Show");
        empty_box.removeClass("empty_servers_visible");
        empty_box.addClass("empty_servers_hidden");
        
        var selected_server = $("#selected_server");
        if (selected_server) {
            var users_online = parseInt($(".users_online", selected_server).text());
            if (users_online <= 0)
                hide_details()
        }
    }
    
    move_empty_servers_box();
}

function update_server_counts(old_users_online, users_online) {
    // Update the players count
    var total_players_node = $("#total_number_players");
    total_players_node.text(parseInt(total_players_node.text()) + (users_online - old_users_online));
    
    // Update the empty servers count
    var change_value = 0;
    if (users_online == 0 && old_users_online == 0)
        change_value = 1;
    else if (users_online > 0 && old_users_online > 0)
        change_value -1;
    
    var number_empty_node = $("#number_empty_servers");
    number_empty_node.text(parseInt(number_empty_node.text()) + change_value);
    
    move_empty_servers_box();
    
    // Update occupied servers count
    change_value = 0;
    if (users_online > 0 && old_users_online == 0)
        change_value = 1;
    else if  (users_online == 0 && old_users_online > 0)
        change_value = -1;
    
    var occupied_servers_node = $("#number_occupied_servers");
    occupied_servers_node.text(parseInt(occupied_servers_node.text()) + change_value);
}

function update_server_node_empty_server_status(server_node, users_online) {
    if (users_online > 0) {
        server_node.removeClass("empty_server");
        server_node.removeClass("hide");
    }
    else {
        server_node.addClass("empty_server");
                
        var empty_box = $("#empty_servers_box");
        if (empty_box.hasClass("empty_servers_hidden")) {
            server_node.addClass("hide");
            
            if (server_node.attr("id") == "selected_server")
                hide_details();
        }
        else if (empty_box.hasClass("empty_servers_visible")) {
            server_node.removeClass("hide");
        }
    }
}

function hide_details() {
    $("#server_details").addClass("hide");
    var selected = $("#selected_server");
    selected.removeClass("highlight");
    selected.attr("id", "");
}

// After the table sorting changes we should still have the details
// box in the correct position
function move_details() {
    var details = $("#server_details");
    if (!details.hasClass("hide"))
        details.insertAfter($("#selected_server"));
}

function update_details(data) {
    var details = $("#server_details");
    var before_node = $("#selected_server");
    details.insertAfter(before_node);
    details.removeClass("hide");
    
    $("#details_options").text(strip_colors(data["options"]));
    $("#details_uri").attr("href", ensure_http(strip_colors(data["uri"])));
    $("#details_uri").text(strip_colors(data["uri"]));
    $("#details_version").text(data["version"]);
    $("#details_connect").attr("href", "armagetronad://" + data["host"] + ":" + data["port"]);
}

function server_info_node(host, port, data) {
    var node = server_node(host, port);
    if (node.length <= 0) {
        node = $("#server_template").clone();
        node.removeClass("hide");
        node.addClass("server");
        node.attr("id", "");
        node.attr("host", host);
        node.attr("port", port);
        
        node.bind("click", function(e) {
            var parent = $(e.target).parent();
            
            if (parent.attr("id") == "selected_server") {
                hide_details();
                return;
            }
                
            $(".highlight").toggleClass("highlight");
            $("#selected_server").attr("id", "");
            parent.toggleClass("highlight");
            parent.attr("id", "selected_server")
            
            update_details(data);
        });
        
        node.hover(
            function() {
                $(this).addClass("hovering");
            },
            function() {
                $(this).removeClass("hovering");
            }
        );
        
        node.appendTo("#servers");
    }
    return node;
}

function update_server_info(host, port, data) {
    data["players"] = $.map(data["players"], function(player, i) {
        player = strip_colors(player);
        if (data["global_ids"][i])
            player += " (" + data["global_ids"][i] + ")";
        return player;
    });
    
    var node = server_info_node(host, port, data);
    var old_users_online = parseInt($(".users_online", node).text());
    var users_online = parseInt(data["num_players_by_server"]);
    
    $(".name", node).text(truncate(strip_colors(data["name"]), 65));
    $(".users_online", node).text(data["num_players_by_server"]);
    $(".users_max", node).text(data["max_players"]);
    $(".names", node).text(data["players"].join(", "))
    
    update_server_node_empty_server_status(node, users_online);        
    update_server_counts(old_users_online, users_online);
    
    $("#servers").trigger("update");
    $("#servers").trigger("sorton", [default_table_sorting()]);
}

function failure_node(host, port) {
    var server = server_node(host, port);
    if (server.length > 0) {
        server.remove();
    }
    
    var node = $("#server_fail_template").clone();
    node.removeClass("hide");
    node.attr("id", "");
    node.attr("host", host);
    node.attr("port", port);
    node.addClass("server");
    node.appendTo("#server_failures");
    
    return node;
}

function update_failed_table(host, port) {
    var node = failure_node(host, port);
     $(".host", node).text(host);
     $(".port", node).text(port);
     
     $("#server_failures").trigger("update");
     $("#server_failures").trigger("sorton", [default_failures_table_sorting()]);
}

function query_server_info(host, port) {
    $.ajax({
        url: "server_info.php",
        dataType : "json",
        data: { host: host, port: port },
        success : function(data, textStatus) {
            update_server_info(host, port, data)
        },
        error : function(x, txt, e) {
            update_failed_table(host, port);
        }
    });
}

function query_all_server_info(data) {
    $.each(data, function(i, server) {
        var host = server["host"];
        var port = server["port"];
        
        query_server_info(host, port);
    });
}

function refresh_all_server_infos() {
    hide_details();
    
    var servers = $("tbody > tr.server");
    $.each(servers, function(i, node) {
        node = $(node);
        var host = node.attr("host");
        var port = node.attr("port");
        query_server_info(host, port);
    });
}

function toggle_unreachable_servers_box() {
    var failures = $("#server_failures");
    var description = $("#server_failures_action_description");
    if (failures.hasClass("hide")) {
        failures.removeClass("hide");
        description.text("Hide");
    }
    else {
        failures.addClass("hide");
        description.text("Show")
    }
}

function default_failures_table_sorting() {
    return [[0, 0], [1, 0]];
}

function update_server_list_cache(data) {
    $.jStorage.set("server_list", data);
    $.jStorage.set("server_list_fetched_date", new Date());
}

function clear_server_list_cache() {
    $.jStorage.flush();
}

function fetch_fresh_server_list() {
    $.ajax({
        url: "server_list.php",
        dataType : "json",
        success : function(data, textStatus) {
            update_server_list_cache(data);
            query_all_server_info(data);
        },
        error : function(x, txt, e) {
            // TODO: better error reporting
            alert("Couldn't get master server list: " + txt);
        }
    });
}

function server_list_old(date) {
    var one_hour = 3600000;
    return (new Date() - date) > one_hour;
}

function fetch_server_list() {
    var data = $.jStorage.get("server_list");
    var date = $.jStorage.get("server_list_fetched_date");
    
    if (!data || !date || server_list_old(date)) {
        fetch_fresh_server_list();
        return;
    }
    
    query_all_server_info(data);    
}

$(document).ready(function () {
    fetch_server_list();
    
    var clear_master_node = $("#clear_server_list_cache");
    clear_master_node.bind("click", function() {
        clear_server_list_cache();
        clear_master_node.text("Master server list cache cleared. Refresh this page to fetch the new list.");
        clear_master_node.removeClass("click");
    });
    
    $(".refresh_all_action").bind("click", function() {
        refresh_all_server_infos();
    });
        
    $.tablesorter.addParser({
        id: "armagetronad-players-online",
        is: function(s) {
            return false;
        },
        format: function(s) {
            return s.replace(/ \/ \d+$/, "");
        },
        type: "numeric"
    });
    
    $.tablesorter.addParser({
        id: "armagetronad-server",
        is: function(s) {
            return false;
        },
        format: function(s) {
            return s.replace(/[^a-zA-Z]/g, "").toLowerCase();
        },
        type: "text"
    });
    
    $("#empty_servers_box").bind("click", function() {
        toggle_empty_servers();
    })
    
    $("#servers").tablesorter({
        sortList: default_table_sorting(),
        headers: {
            0: { sorter: "armagetronad-server" },
            1: { sorter: "armagetronad-players-online" }
        }
    });
    
    $("#servers").bind("sortEnd", function() {
        move_details();
        move_empty_servers_box();
    })
    
    $("#refresh_server").bind("click", function() {
        var parent = $("#selected_server")
        
        var host = parent.attr("host");
        var port = parent.attr("port");
        
        query_server_info(host, port);
    });
    
    $("#server_failures_action").bind("click", function() {
        toggle_unreachable_servers_box();
    });
    
    $("#server_failures").tablesorter({
        sortList: default_failures_table_sorting()
    });
});
